from getpass import getpass

from .db import PVaultDB


def auth(db: PVaultDB):
    if not db.exists():
        db.create_database()

        master = getpass("Set master password: ")
        master_conf = getpass("Confirm master password: ")

        while master != master_conf and master != "":
            print("Passwords don't match.")
            master = getpass("Set master password: ")
            master_conf = getpass("Confirm master password: ")

        db.encrypt(master_conf)
        db.decrypt(master_conf)
        return master

    else:
        master = getpass("Enter master password: ")
        db.decrypt(master)
        return master



import pyperclip
from getpass import getpass

from .generator import generate_password
from .crypto import decrypt_password


def confirmation(account, action):
    if action == "r":
        response = input("Are you sure you want to regenerate password for"
                         " '{}' (y/n): ".format(account))
    elif action == "d":
        response = input("Are you sure you want to delete password for"
                         " '{}' (y/n): ".format(account))
    elif action == "u":
        response = input("Are you sure you want to update password for"
                         " '{}' (y/n): ".format(account))
    elif action == "da":
        response = input("Are you sure you want to delete all saved passwords "
                         "(y/n): ")
    return response.lower() == "y"


def confirmation_message(account, action):
    if action == "g":
        print("Password generated for account: '{}'".format(account) +
              "\nCopied to clipboard!")

    if action == "r":
        print("Password regenerated for account: '{}''".format(account) +
              "\nCopied to clipboard!")


def generate(command, db, auth):
    if len(command) > 1:
        account = command[1]
        password = generate_password()
        if not db.query(account):
            db.save_password(account, password, auth)
            pyperclip.copy(password)
            confirmation_message(account, "g")
        else:
            if confirmation(account, "r"):
                db.update_password(account, password, auth)
                pyperclip.copy(password)
                confirmation_message(account, "r")
    else:
        pyperclip.copy(generate_password())
        print("Random password copied to clipboard!")


def get(command, db, auth):
    if len(command) > 1:
        account = command[1]
        password = db.query(account)[2]
        password = decrypt_password(password, auth)
        pyperclip.copy(password)
        print("Password copied to clipboard for '{}'!".format(account))

    elif len(command) == 1:
        accounts = db.get_all_accounts()
        if len(accounts) > 0:
            print("Accounts with passwords saved are:\n" +
                  "\n".join(accounts))
        else:
            print("No accounts saved.")


def delete(command, db, auth):
    if len(command) > 1:
        account = command[1]
        if confirmation(account, "d"):
            db.delete_account(account)
            print("'{}' deleted.".format(account))

    elif len(command) == 1:
        if confirmation("all", "da"):
            db.delete_all_accounts()
            print("All accounts deleted.")


def save(command, db, auth):
    if len(command) > 1:
        account = command[1]
        password = getpass("Enter password for '{}': ".format(account))
        if not db.query(account):
            db.save_password(account, password, auth)
            pyperclip.copy(password)
            print("Password saved for '{}'!".format(account))
        else:
            if confirmation(account, "u"):
                db.update_password(account, password, auth)
                pyperclip.copy(password)
                print("Password updated for '{}'!".format(account))
    else:
        print("Account name required.")


def help_me(command, db):
    messages = [
        "usage: pv [command]",
        "A python CLI based password manager",
        "generate  generates a random password to your clipboard",
        "          [accountname] sets a unique identifier for your password and save; overwrites if exists",
        "accounts  shows all saved passwords",
        "account   [accountname] shows the password of a particular user only",
        "save      [accountname] awaits password input and binds to accountname",
        "delete    delete all passwords in the database",
        "          [accountname] delete only this user in the database"
    ]
    for message in messages: print(message)



import sqlite3
import os

from pyAesCrypt import encryptFile, decryptFile

from .config import __dbPath__, __dbName__
from .crypto import encrypt_password


class EncryptionError(PermissionError):
    pass


class PVaultDB:
    def __init__(self):
        self.name = __dbName__
        self.directory = os.path.dirname(__dbPath__)

    def exists(self):
        return __dbName__ in os.listdir(self.directory) \
            or __dbName__+'.enc' in os.listdir(self.directory)

    def is_decrypted(self):
        if self.exists():
            return __dbName__ in os.listdir(self.directory)

    def encrypt(self, master):
        encryptFile(__dbPath__, __dbPath__ + '.enc', master, 64*1024)
        os.remove(__dbPath__)

    def decrypt(self, master):
        decryptFile(__dbPath__ + '.enc', __dbPath__, master, 64*1024)
        os.remove(__dbPath__ + '.enc')

    def create_database(self):
        """create database upon initialization"""
        with sqlite3.connect(__dbPath__) as conn:
            c = conn.cursor()
            cmd = """CREATE TABLE IF NOT EXISTS passwords (
                        id integer PRIMARY KEY,
                        account text NOT NULL,
                        password text NOT NULL
                    );
                """
            c.execute(cmd)

    def query(self, account):
        if not self.is_decrypted():
            raise EncryptionError('DB is encrypted')
        with sqlite3.connect(__dbPath__) as conn:
            c = conn.cursor()
            c.execute("SELECT * FROM passwords WHERE account=?", (account,))
            return c.fetchone()

    def get_all_accounts(self):
        if not self.is_decrypted():
            raise EncryptionError('DB is encrypted')
        with sqlite3.connect(__dbPath__) as conn:
            c = conn.cursor()
            c.execute("SELECT * FROM passwords")
            return [item[1] for item in c.fetchall()]

    def save_password(self, account, password, auth):
        password = encrypt_password(password, auth)
        if not self.is_decrypted():
            raise EncryptionError('DB is encrypted')
        """save generated password to account in db"""
        with sqlite3.connect(__dbPath__) as conn:
            c = conn.cursor()
            cmd = """INSERT INTO passwords(account, password)
                    VALUES (?, ?)"""
            c.execute(cmd, (account, password))
        return c.lastrowid

    def update_password(self, account, new_password, auth):
        new_password = encrypt_password(new_password, auth)
        if not self.is_decrypted():
            raise EncryptionError('DB is encrypted')
        """updates an account with a new password"""
        with sqlite3.connect(__dbPath__) as conn:
            c = conn.cursor()
            _id = self.query(account)[0]
            cmd = """UPDATE passwords
                    SET password = ?
                    WHERE id = ?"""
            c.execute(cmd, (new_password, _id))

    def delete_account(self, account):
        if not self.is_decrypted():
            raise EncryptionError('DB is encrypted')
        """deletes entries for an account"""
        with sqlite3.connect(__dbPath__) as conn:
            c = conn.cursor()
            _id = self.query(account)[0]
            cmd = "DELETE FROM passwords WHERE id = ?"
            if account == "all":
                cmd = "DELETE FROM passwords"
                c.execute(cmd)
                return
            c.execute(cmd, (_id,))

    def delete_all_accounts(self):
        if not self.is_decrypted():
            raise EncryptionError('DB is encrypted')
        with sqlite3.connect(__dbPath__) as conn:
            c = conn.cursor()
            c.execute("DELETE FROM passwords")
            self.create_database()



import base64
from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC


def gen_key(master):
    password = master.encode()
    salt = master.encode()
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=100000,
        backend=default_backend()
    )
    return base64.urlsafe_b64encode(kdf.derive(password))


def encrypt_password(password, master):
    f = Fernet(gen_key(master))
    return f.encrypt(password.encode())


def decrypt_password(password, master):
    f = Fernet(gen_key(master))
    return f.decrypt(password).decode()



from platform import system
import os


__dbName__ = 'pvault.db'
__dbPath__ = ''
__ver__ = "1.0.0"

if system() == "Windows":
    __dbPath__ = os.path.join(os.getenv('APPDATA'), __dbName__)

if system() == "Linux":
    __dbPath__ = os.path.join(os.getenv('HOME'), 'bin', __dbName__)



import setuptools

with open('requirements.txt') as f:
    requirements = f.readlines()

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pvault",
    version="1.1.3a",
    author="oluwafenyi",
    author_email="o.enyioma@gmail.com",
    description="A password manager package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/oluwafenyi/password-vault",
    download_url="https://pypi.org/project/pvault/",
    license="MIT",
    keywords="password crypto management",
    packages=setuptools.find_packages(exclude=["contrib", "docs", "tests"]),
    python_requires='>=3.5',
    package_name=["pvault"],
    entry_points={
        'console_scripts': [
            'pv = pvault.pv:main'
        ]
    },
    scripts=[
        'pvault/pv.py'
    ],
    include_package_data=True,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux"
    ],
    install_requires=requirements
)



#!/usr/bin/env python
import sys
import os

from . import handler
from .db import PVaultDB
from .auth import auth


def main():
    db = PVaultDB()

    try:
        if sys.argv[1] == "help":
            handler.help_me(sys.argv[1], db)
            os._exit(1)

        elif sys.argv[1] == "generate" and len(sys.argv) == 2:
            handler.generate(sys.argv[1:], db, None)
            os._exit(1)

        authenticated = None

        try:
            authenticated = auth(db)
        except ValueError:
            print("Wrong Password")  # or db is corrupt

        if authenticated:
            command = sys.argv[1:]
            if not command:
                print("Invalid Argument\n\n\n")
                handler.help_me(None, db)

            switcher = {
                "generate": handler.generate,
                "account": handler.get,
                "accounts": handler.get,
                "delete": handler.delete,
                "save": handler.save,
            }

            event = switcher.get(command[0], "error")

            if not isinstance(event, str):
                event(command, db, authenticated)
            else:
                print("Invalid Argument\n\n\n")
                handler.help_me(None, db)

            db.encrypt(authenticated)

    except:
        print("An error has occurred.")
        db.encrypt(authenticated)
